@@ -51,21 +51,6 @@
 
 static void
 thunar_dialogs_select_filename (GtkWidget *entry);
-static void
-thunar_dialogs_shrink_height (GtkWidget *dialog);
-
-
-
-static void
-thunar_dialogs_shrink_height (GtkWidget *dialog)
-{
-  gint width;
-  gint height;
-
-  /* Shrinks the dialog to it's minimum height */
-  gtk_window_get_size (GTK_WINDOW (dialog), &width, &height);
-  gtk_window_resize (GTK_WINDOW (dialog), width, 1);
-}
 
 
 
@@ -75,6 +60,8 @@ thunar_dialogs_shrink_height (GtkWidget *dialog)
  * @content_type : the content type of the file or folder to create.
  * @filename     : the suggested filename or %NULL.
  * @title        : the dialog title.
+ * @startup_id   : startup id from startup notification passed along
+ *                 with dbus to make focus stealing work properly. Ignored if NULL.
  *
  * Displays a Thunar create dialog  with the specified
  * parameters that asks the user to enter a name for a new file or
@@ -90,11 +77,11 @@ gchar *
 thunar_dialogs_show_create (gpointer     parent,
                             const gchar *content_type,
                             const gchar *filename,
-                            const gchar *title)
+                            const gchar *title,
+                            const gchar *startup_id)
 {
   GtkWidget         *dialog;
-  GtkWindow         *window = NULL;
-  GdkScreen         *screen;
+  GtkWindow         *window;
   GError            *error = NULL;
   gchar             *name = NULL;
   GtkWidget         *label;
@@ -106,8 +93,8 @@ thunar_dialogs_show_create (gpointer     parent,
 
   _thunar_return_val_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent), NULL);
 
-  /* parse the parent window and screen */
-  screen = thunar_util_parse_parent (parent, &window);
+  /* parse the parent window */
+  window = thunar_util_find_associated_window (parent);
 
   /* create a new dialog window */
   dialog = gtk_dialog_new_with_buttons (title,
@@ -118,6 +105,14 @@ thunar_dialogs_show_create (gpointer     parent,
                                           _("C_reate"), GTK_RESPONSE_OK,
                                         NULL);
 
+  /* If a specific screen is requested, we spawn the dialog on that screen */
+  if (G_UNLIKELY (parent != NULL && GDK_IS_SCREEN (parent)))
+    thunar_gtk_window_set_screen (GTK_WINDOW (dialog), GDK_SCREEN (parent));
+
+  /* set the startup id */
+  if (startup_id != NULL)
+    gtk_window_set_startup_id (GTK_WINDOW (dialog), startup_id);
+
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
   gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_OK, FALSE);
   gtk_window_set_default_size (GTK_WINDOW (dialog), 300, -1);
@@ -174,15 +169,6 @@ thunar_dialogs_show_create (gpointer     parent,
   /* select the filename without the extension */
   thunar_dialogs_select_filename (GTK_WIDGET (xfce_filename_input_get_entry (filename_input)));
 
-  if (screen != NULL)
-    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
-
-  if (window != NULL)
-    gtk_window_set_transient_for (GTK_WINDOW (dialog), window);
-
-  /* shrink the dialog again, after some filename error was fixed */
-  g_signal_connect_swapped (filename_input, "text-valid", G_CALLBACK (thunar_dialogs_shrink_height), dialog);
-
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
     {
       /* determine the chosen filename */
@@ -210,10 +196,12 @@ thunar_dialogs_show_create (gpointer     parent,
 
 /**
  * thunar_dialogs_show_rename_file:
- * @parent : a #GtkWidget on which the error dialog should be shown, or a #GdkScreen
- *           if no #GtkWidget is known. May also be %NULL, in which case the default
- *           #GdkScreen will be used.
- * @file   : the #ThunarFile we're going to rename.
+ * @parent      : a #GtkWidget on which the error dialog should be shown, or a #GdkScreen
+ *                if no #GtkWidget is known. May also be %NULL, in which case the default
+ *                #GdkScreen will be used.
+ * @file        : the #ThunarFile we're going to rename.
+ * @startup_id  : startup id from startup notification passed along
+ *                with dbus to make focus stealing work properly. Ignored if NULL.
  *
  * Displays the Thunar rename dialog for a single file rename.
  *
@@ -223,12 +211,13 @@ thunar_dialogs_show_create (gpointer     parent,
 ThunarJob *
 thunar_dialogs_show_rename_file (gpointer               parent,
                                  ThunarFile            *file,
+                                 const gchar           *startup_id,
                                  ThunarOperationLogMode log_mode)
 {
   ThunarIconFactory *icon_factory;
   GtkIconTheme      *icon_theme;
   gint               scale_factor;
-  const gchar       *filename;
+  gchar             *filename;
   const gchar       *text;
   ThunarJob         *job = NULL;
   GtkWidget         *dialog;
@@ -237,8 +226,7 @@ thunar_dialogs_show_rename_file (gpointer               parent,
   GtkWidget         *grid;
   XfceFilenameInput *filename_input;
   GtkEntry          *filename_input_entry;
-  GtkWindow         *window = NULL;
-  GdkScreen         *screen;
+  GtkWindow         *window;
   GdkPixbuf         *icon;
   cairo_surface_t   *surface;
   gchar             *title;
@@ -250,11 +238,12 @@ thunar_dialogs_show_rename_file (gpointer               parent,
   _thunar_return_val_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WINDOW (parent), FALSE);
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
 
-  /* parse the parent window and screen */
-  screen = thunar_util_parse_parent (parent, &window);
+  /* parse the parent window */
+  window = thunar_util_find_associated_window (parent);
 
-  /* get the filename of the file */
-  filename = thunar_file_get_basename (file);
+  /* Get the filename of the file. We create our own copy of the string because the file
+   * can be modified by another process while the rename is in progress (issue #1639). */
+  filename = g_strdup (thunar_file_get_basename (file));
 
   /* create a new dialog window */
   title = g_strdup_printf (_("Rename \"%s\""), filename);
@@ -268,9 +257,13 @@ thunar_dialogs_show_rename_file (gpointer               parent,
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
   g_free (title);
 
-  /* move the dialog to the appropriate screen */
-  if (G_UNLIKELY (window == NULL && screen != NULL))
-    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+  /* If a specific screen is requested, we spawn the dialog on that screen */
+  if (G_UNLIKELY (parent != NULL && GDK_IS_SCREEN (parent)))
+    thunar_gtk_window_set_screen (GTK_WINDOW (dialog), GDK_SCREEN (parent));
+
+  /* set the startup id */
+  if (startup_id != NULL)
+    gtk_window_set_startup_id (GTK_WINDOW (dialog), startup_id);
 
   if (window != NULL)
     scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (window));
@@ -353,15 +346,19 @@ thunar_dialogs_show_rename_file (gpointer               parent,
   /* resize the dialog to make long names fit as much as possible */
   gtk_window_set_default_size (GTK_WINDOW (dialog), CLAMP (dialog_width, 300, parent_width), -1);
 
-  /* automatically close the dialog when the file is destroyed */
+  /* automatically close the dialog when the file is renamed externally or destroyed */
+  g_signal_connect_swapped (G_OBJECT (file), "renamed",
+                            G_CALLBACK (gtk_widget_destroy), dialog);
   g_signal_connect_swapped (G_OBJECT (file), "destroy",
                             G_CALLBACK (gtk_widget_destroy), dialog);
 
-  /* shrink the dialog again, after some filename error was fixed */
-  g_signal_connect_swapped (filename_input, "text-valid", G_CALLBACK (thunar_dialogs_shrink_height), dialog);
-
   /* run the dialog */
   response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+  /* unregister handlers */
+  g_signal_handlers_disconnect_by_func (G_OBJECT (file), gtk_widget_destroy, dialog);
+
+  /* handle the dialog response */
   if (G_LIKELY (response == GTK_RESPONSE_OK))
     {
       /* hide the dialog */
@@ -381,12 +378,9 @@ thunar_dialogs_show_rename_file (gpointer               parent,
 
   /* cleanup */
   if (G_LIKELY (response != GTK_RESPONSE_NONE))
-    {
-      /* unregister handler */
-      g_signal_handlers_disconnect_by_func (G_OBJECT (file), gtk_widget_destroy, dialog);
+    gtk_widget_destroy (dialog);
 
-      gtk_widget_destroy (dialog);
-    }
+  g_free (filename);
 
   return job;
 }
@@ -485,8 +479,7 @@ thunar_dialogs_show_error (gpointer      parent,
                            ...)
 {
   GtkWidget *dialog;
-  GtkWindow *window = NULL;
-  GdkScreen *screen;
+  GtkWindow *window;
   va_list    args;
   gchar     *primary_text;
   GList     *children;
@@ -499,7 +492,7 @@ thunar_dialogs_show_error (gpointer      parent,
     return;
 
   /* parse the parent pointer */
-  screen = thunar_util_parse_parent (parent, &window);
+  window = thunar_util_find_associated_window (parent);
 
   /* determine the primary error text */
   va_start (args, format);
@@ -516,9 +509,9 @@ thunar_dialogs_show_error (gpointer      parent,
 
   gtk_window_set_title (GTK_WINDOW (dialog), _("Error"));
 
-  /* move the dialog to the appropriate screen */
-  if (G_UNLIKELY (window == NULL && screen != NULL))
-    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+  /* If a specific screen is requested, we spawn the dialog on that screen */
+  if (G_UNLIKELY (parent != NULL && GDK_IS_SCREEN (parent)))
+    thunar_gtk_window_set_screen (GTK_WINDOW (dialog), GDK_SCREEN (parent));
 
   /* set secondary text if an error is provided */
   if (G_LIKELY (error != NULL))
@@ -561,7 +554,8 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
 {
   const gchar *separator;
   const gchar *mnemonic;
-  GtkWidget   *message;
+  GtkWindow   *window;
+  GtkWidget   *dialog;
   GtkWidget   *button;
   GString     *secondary = g_string_sized_new (256);
   GString     *primary = g_string_sized_new (256);
@@ -572,6 +566,9 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
   _thunar_return_val_if_fail (parent == NULL || GTK_IS_WINDOW (parent), THUNAR_JOB_RESPONSE_CANCEL);
   _thunar_return_val_if_fail (g_utf8_validate (question, -1, NULL), THUNAR_JOB_RESPONSE_CANCEL);
 
+  /* parse the parent pointer */
+  window = thunar_util_find_associated_window (parent);
+
   /* try to separate the question into primary and secondary parts */
   separator = strstr (question, ": ");
   if (G_LIKELY (separator != NULL))
@@ -608,14 +605,19 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
     }
 
   /* allocate the question message dialog */
-  message = gtk_message_dialog_new (parent,
-                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-                                    GTK_MESSAGE_QUESTION,
-                                    GTK_BUTTONS_NONE,
-                                    "%s", primary->str);
-  gtk_window_set_title (GTK_WINDOW (message), _("Attention"));
+  dialog = gtk_message_dialog_new (window,
+                                   GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+                                   GTK_MESSAGE_QUESTION,
+                                   GTK_BUTTONS_NONE,
+                                   "%s", primary->str);
+
+  /* If a specific screen is requested, we spawn the dialog on that screen */
+  if (G_UNLIKELY (parent != NULL && GDK_IS_SCREEN (parent)))
+    thunar_gtk_window_set_screen (GTK_WINDOW (dialog), GDK_SCREEN (parent));
+
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Attention"));
   if (G_LIKELY (*secondary->str != '\0'))
-    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message), "%s", secondary->str);
+    gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", secondary->str);
 
   /* add the buttons based on the possible choices */
   for (n = THUNAR_JOB_RESPONSE_MAX_INT; n >= 0; --n)
@@ -687,24 +689,24 @@ thunar_dialogs_show_job_ask (GtkWindow        *parent,
 
       button = gtk_button_new_with_mnemonic (mnemonic);
       gtk_widget_set_can_default (button, TRUE);
-      gtk_dialog_add_action_widget (GTK_DIALOG (message), button, response);
+      gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, response);
       gtk_widget_show (button);
 
-      gtk_dialog_set_default_response (GTK_DIALOG (message), response);
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog), response);
     }
 
   if (has_cancel)
     {
       button = gtk_button_new_with_mnemonic (_("_Cancel"));
       gtk_widget_set_can_default (button, TRUE);
-      gtk_dialog_add_action_widget (GTK_DIALOG (message), button, GTK_RESPONSE_CANCEL);
+      gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, GTK_RESPONSE_CANCEL);
       gtk_widget_show (button);
-      gtk_dialog_set_default_response (GTK_DIALOG (message), GTK_RESPONSE_CANCEL);
+      gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
     }
 
   /* run the question dialog */
-  response = gtk_dialog_run (GTK_DIALOG (message));
-  gtk_widget_destroy (message);
+  response = gtk_dialog_run (GTK_DIALOG (dialog));
+  gtk_widget_destroy (dialog);
 
   /* transform the result as required */
   if (G_UNLIKELY (response <= 0))
@@ -1043,8 +1045,7 @@ thunar_dialogs_show_insecure_program (gpointer     parent,
                                       ThunarFile  *file,
                                       const gchar *command)
 {
-  GdkScreen *screen;
-  GtkWindow *window = NULL;
+  GtkWindow *window;
   gint       response;
   GtkWidget *dialog;
   GString   *secondary;
@@ -1055,8 +1056,8 @@ thunar_dialogs_show_insecure_program (gpointer     parent,
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
   _thunar_return_val_if_fail (g_utf8_validate (command, -1, NULL), FALSE);
 
-  /* parse the parent window and screen */
-  screen = thunar_util_parse_parent (parent, &window);
+  /* parse the parent window */
+  window = thunar_util_find_associated_window (parent);
 
   /* create the secondary text */
   if (g_strcmp0 (thunar_file_get_display_name (file), thunar_file_get_basename (file)) != 0)
@@ -1092,14 +1093,17 @@ thunar_dialogs_show_insecure_program (gpointer     parent,
                                    GTK_MESSAGE_WARNING,
                                    GTK_BUTTONS_NONE,
                                    "%s", primary);
+
+  /* If a specific screen is requested, we spawn the dialog on that screen */
+  if (G_UNLIKELY (parent != NULL && GDK_IS_SCREEN (parent)))
+    thunar_gtk_window_set_screen (GTK_WINDOW (dialog), GDK_SCREEN (parent));
+
   gtk_window_set_title (GTK_WINDOW (dialog), _("Attention"));
   gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Launch Anyway"), GTK_RESPONSE_OK);
   if (thunar_file_is_chmodable (file))
     gtk_dialog_add_button (GTK_DIALOG (dialog), _("Mark As _Secure And Launch"), GTK_RESPONSE_APPLY);
   gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Cancel"), GTK_RESPONSE_CANCEL);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CANCEL);
-  if (screen != NULL && window == NULL)
-    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), "%s", secondary->str);
   g_string_free (secondary, TRUE);
   response = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -1239,8 +1243,7 @@ thunar_dialog_ask_execute (const ThunarFile *file,
                            gboolean          single_file)
 {
   GtkWidget *dialog;
-  GtkWindow *window = NULL;
-  GdkScreen *screen;
+  GtkWindow *window;
   gint       response;
   gchar     *dialog_text;
   GtkWidget *button;
@@ -1248,8 +1251,8 @@ thunar_dialog_ask_execute (const ThunarFile *file,
   _thunar_return_val_if_fail (THUNAR_IS_FILE (file), FALSE);
   _thunar_return_val_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent), THUNAR_FILE_ASK_EXECUTE_RESPONSE_OPEN);
 
-  /* parse the parent window and screen */
-  screen = thunar_util_parse_parent (parent, &window);
+  /* parse the parent window */
+  window = thunar_util_find_associated_window (parent);
 
   if (single_file)
     {
@@ -1268,6 +1271,10 @@ thunar_dialog_ask_execute (const ThunarFile *file,
                                    GTK_BUTTONS_NONE,
                                    "%s", dialog_text);
 
+  /* If a specific screen is requested, we spawn the dialog on that screen */
+  if (G_UNLIKELY (parent != NULL && GDK_IS_SCREEN (parent)))
+    thunar_gtk_window_set_screen (GTK_WINDOW (dialog), GDK_SCREEN (parent));
+
   gtk_window_set_title (GTK_WINDOW (dialog), _("Open Shell Script"));
 
   g_free (dialog_text);
@@ -1295,9 +1302,6 @@ thunar_dialog_ask_execute (const ThunarFile *file,
       gtk_dialog_set_default_response (GTK_DIALOG (dialog), THUNAR_FILE_ASK_EXECUTE_RESPONSE_RUN);
     }
 
-  if (G_UNLIKELY (window == NULL && screen != NULL))
-    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
-
   response = gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
 
@@ -1327,7 +1331,7 @@ thunar_dialog_show_launcher_props (ThunarFile *launcher,
   _thunar_return_if_fail (launcher == NULL || THUNAR_IS_FILE (launcher));
   _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
 
-  screen = thunar_util_parse_parent (parent, NULL);
+  screen = thunar_util_parse_parent (parent);
   uri = thunar_file_dup_uri (launcher);
   display_name = gdk_display_get_name (gdk_screen_get_display (screen));
   cmd = g_strdup_printf ("xfce-desktop-item-edit \"--display=%s\" \"%s\"", display_name, uri);
