@@ -323,8 +323,8 @@ thunar_zoom_level_to_view_margin (ThunarZoomLevel zoom_level) G_GNUC_CONST;
 const gchar *
 thunar_zoom_level_string_from_value (ThunarZoomLevel zoom_level);
 gboolean
-thunar_zoom_level_value_from_string (const gchar *value_string,
-                                     gint        *value);
+thunar_zoom_level_value_from_string (const gchar     *value_string,
+                                     ThunarZoomLevel *value);
 
 
 #define THUNAR_TYPE_JOB_RESPONSE (thunar_job_response_get_type ())
