@@ -103,6 +103,8 @@ thunar_folder_monitor (GFileMonitor     *monitor,
                        GFileMonitorEvent event_type,
                        gpointer          user_data);
 static void
+thunar_folder_reset_monitor (ThunarFolder *folder);
+static void
 thunar_folder_load_content_types (ThunarFolder *folder,
                                   GHashTable   *files);
 static void
@@ -151,10 +153,10 @@ struct _ThunarFolder
   /* Files which were changed recently. The key is a ThunarFile; value is NULL (unimportant)*/
   GHashTable *changed_files_map;
 
-  /* Files which were changed recently. The key is a ThunarFile; value is NULL (unimportant)*/
+  /* Files which were added recently. The key is a ThunarFile; value is NULL (unimportant)*/
   GHashTable *added_files_map;
 
-  /* Files which were changed recently. The key is a ThunarFile; value is NULL (unimportant)*/
+  /* Files which were removed recently. The key is a ThunarFile; value is NULL (unimportant)*/
   GHashTable *removed_files_map;
 
   /* Files inside this folder. The key is a ThunarFile; value is NULL (unimportant)*/
@@ -194,18 +196,8 @@ static void
 thunar_folder_constructed (GObject *object)
 {
   ThunarFolder *folder = THUNAR_FOLDER (object);
-  GError       *error = NULL;
 
-  folder->monitor = g_file_monitor_directory (thunar_file_get_file (folder->corresponding_file),
-                                              G_FILE_MONITOR_WATCH_MOVES, NULL, &error);
-
-  if (G_LIKELY (folder->monitor != NULL))
-    g_signal_connect (folder->monitor, "changed", G_CALLBACK (thunar_folder_monitor), folder);
-  else
-    {
-      g_debug ("Could not create folder monitor: %s", error->message);
-      g_error_free (error);
-    }
+  thunar_folder_reset_monitor (folder);
 
   G_OBJECT_CLASS (thunar_folder_parent_class)->constructed (object);
 }
@@ -596,17 +588,25 @@ _thunar_folder_files_update_timeout (gpointer data)
   g_hash_table_remove_all (files);
   g_hash_table_remove_all (folder->added_files_map);
 
-  /* send a 'changed' signal for all files which changed */
+  /* reload files which were changed and send a 'changed' signal for them, if required */
   g_hash_table_iter_init (&iter, folder->changed_files_map);
   while (g_hash_table_iter_next (&iter, &key, NULL))
     {
+      /* block 'changed' signals for this file until reload is done, in order to prevent recursion */
+      g_signal_handlers_block_by_func (G_OBJECT (key), G_CALLBACK (thunar_folder_file_changed), folder);
+      thunar_file_reload (THUNAR_FILE (key));
+      g_signal_handlers_unblock_by_func (G_OBJECT (key), G_CALLBACK (thunar_folder_file_changed), folder);
+
+      /* only send the 'changed' signal for files which are already part of this folder */
       if (!g_hash_table_contains (folder->files_map, key))
         continue;
 
       g_hash_table_add (files, g_object_ref (key));
     }
 
-  g_signal_emit (G_OBJECT (folder), folder_signals[FILES_CHANGED], 0, files);
+  if (g_hash_table_size (files) > 0)
+    g_signal_emit (G_OBJECT (folder), folder_signals[FILES_CHANGED], 0, files);
+
   g_hash_table_destroy (files);
   g_hash_table_remove_all (folder->changed_files_map);
 
@@ -647,6 +647,10 @@ thunar_folder_add_file (ThunarFolder *folder,
   if (g_hash_table_remove (folder->removed_files_map, file))
     return;
 
+  /* Do nothing if we already have the file in our internal map */
+  if (g_hash_table_lookup (folder->files_map, file) != NULL)
+    return;
+
   g_hash_table_add (folder->added_files_map, g_object_ref (file));
 
   if (folder->files_update_timeout_source_id == 0)
@@ -663,6 +667,10 @@ thunar_folder_remove_file (ThunarFolder *folder,
   if (g_hash_table_remove (folder->added_files_map, file))
     return;
 
+  /* Do nothing if we dont have the file yet in our internal map */
+  if (g_hash_table_lookup (folder->files_map, file) == NULL)
+    return;
+
   g_hash_table_add (folder->removed_files_map, g_object_ref (file));
 
   if (folder->files_update_timeout_source_id == 0)
@@ -959,9 +967,9 @@ thunar_folder_monitor (GFileMonitor     *monitor,
 
   if (g_file_equal (event_file, thunar_file_get_file (folder->corresponding_file)))
     {
-      /* update/destroy the corresponding file */
+      /* update the corresponding file or signal destruction */
       if (event_type == G_FILE_MONITOR_EVENT_DELETED)
-        thunar_file_destroy (folder->corresponding_file);
+        thunar_file_signal_destroy (folder->corresponding_file);
       else
         thunar_file_reload (folder->corresponding_file);
       return;
@@ -1009,8 +1017,8 @@ thunar_folder_monitor (GFileMonitor     *monitor,
       /* Drop it from the map, if we still have it */
       thunar_folder_remove_file (folder, event_file_thunar);
 
-      /* destroy the old file */
-      thunar_file_destroy (event_file_thunar);
+      /* signal destruction of the old file */
+      thunar_file_signal_destroy (event_file_thunar);
 
       break;
 
@@ -1052,9 +1060,9 @@ thunar_folder_monitor (GFileMonitor     *monitor,
       /* re-add the renamed file into our internal map (this time we can use the timeout source) */
       thunar_folder_add_file (folder, renamed_file);
 
-      /* destroy the old ThunarFile, if we have two of them */
+      /* Signal destruction for the old ThunarFile, if we have two of them */
       if (event_file_thunar != NULL && other_file_thunar != NULL)
-        thunar_file_destroy (event_file_thunar);
+        thunar_file_signal_destroy (event_file_thunar);
 
       /* reload the renamed file */
       if (thunar_file_reload (renamed_file))
@@ -1070,16 +1078,12 @@ thunar_folder_monitor (GFileMonitor     *monitor,
     case G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED:
       if (event_file_thunar != NULL)
         {
-          /* If we have that file in our internal map, reload it and send a file-changed signal */
-          if (g_hash_table_lookup (folder->files_map, event_file_thunar) != NULL)
-            {
-              thunar_folder_file_changed (folder, event_file_thunar);
+          /* this will trigger a reload if the file and send a 'file-changed' signal */
+          thunar_folder_file_changed (folder, event_file_thunar);
 
 #if DEBUG_FILE_CHANGES
-              thunar_file_infos_equal (event_file_thunar, event_file);
+          thunar_file_infos_equal (event_file_thunar, event_file);
 #endif
-              thunar_file_reload (event_file_thunar);
-            }
         }
 
     default:
@@ -1100,6 +1104,32 @@ thunar_folder_monitor (GFileMonitor     *monitor,
 
 
 
+static void
+thunar_folder_reset_monitor (ThunarFolder *folder)
+{
+  GError *error = NULL;
+
+  if (folder->monitor != NULL)
+    {
+      g_signal_handlers_disconnect_by_data (folder->monitor, folder);
+      g_file_monitor_cancel (folder->monitor);
+      g_object_unref (folder->monitor);
+    }
+
+  folder->monitor = g_file_monitor_directory (thunar_file_get_file (folder->corresponding_file),
+                                              G_FILE_MONITOR_WATCH_MOVES, NULL, &error);
+
+  if (G_LIKELY (folder->monitor != NULL))
+    g_signal_connect (folder->monitor, "changed", G_CALLBACK (thunar_folder_monitor), folder);
+  else
+    {
+      g_debug ("Could not create folder monitor: %s", error->message);
+      g_error_free (error);
+    }
+}
+
+
+
 /**
  * thunar_folder_get_for_file:
  * @file : a #ThunarFile.
@@ -1274,6 +1304,9 @@ thunar_folder_reload (ThunarFolder *folder,
   g_signal_connect (folder->job, "finished", G_CALLBACK (thunar_folder_finished), folder);
   g_signal_connect (folder->job, "files-ready", G_CALLBACK (thunar_folder_files_ready), folder);
   thunar_job_launch (THUNAR_JOB (folder->job));
+
+  /* reset the monitoring */
+  thunar_folder_reset_monitor (folder);
 }
 
 
