@@ -103,6 +103,7 @@ thunar_gtk_menu_thunarx_menu_item_new (GObject      *thunarx_menu_item,
   gchar       *name, *label_text, *tooltip_text, *icon_name, *accel_path;
   gboolean     sensitive;
   GtkWidget   *gtk_menu_item;
+  GtkWidget   *gtk_submenu_item;
   ThunarxMenu *thunarx_menu;
   GList       *children;
   GList       *lp;
@@ -138,9 +139,14 @@ thunar_gtk_menu_thunarx_menu_item_new (GObject      *thunarx_menu_item,
       children = thunarx_menu_get_items (thunarx_menu);
       submenu = gtk_menu_new ();
       for (lp = children; lp != NULL; lp = lp->next)
-        thunar_gtk_menu_thunarx_menu_item_new (lp->data, GTK_MENU_SHELL (submenu));
+        {
+          gtk_submenu_item = thunar_gtk_menu_thunarx_menu_item_new (lp->data, GTK_MENU_SHELL (submenu));
+
+          /* Each thunarx_menu_item will be destroyed together with its related gtk_menu_item */
+          g_signal_connect_swapped (G_OBJECT (gtk_submenu_item), "destroy", G_CALLBACK (g_object_unref), lp->data);
+        }
       gtk_menu_item_set_submenu (GTK_MENU_ITEM (gtk_menu_item), submenu);
-      thunarx_menu_item_list_free (children);
+      g_list_free (children);
     }
 
   g_free (name);
@@ -213,12 +219,13 @@ thunar_gtk_menu_popup_at_pointer (GtkMenu  *menu,
                                   GdkEvent *event)
 {
   GdkWindow   *window;
-  GdkWindow   *toplevel = NULL;
   GdkDevice   *device;
+  GdkDisplay  *display;
+  GdkSeat     *seat;
   GdkRectangle rect = { 0, 0, 1, 1 };
 
-  /* fallback if event not set or event not holding a GdkSeat */
-  if (event == NULL || gdk_event_get_seat (event) == NULL)
+  /* fallback if event not set */
+  if (event == NULL)
     {
       gtk_menu_popup_at_pointer (menu, event);
       return;
@@ -228,25 +235,38 @@ thunar_gtk_menu_popup_at_pointer (GtkMenu  *menu,
   window = gdk_event_get_window (event);
   if (window != NULL)
     {
-      /* Ensure that we have the toplevel window to work around a GTK3 Wayland bug
-       * that prevents the user from dismissing the menu via click in some cases.
-       * See: https://gitlab.xfce.org/xfce/thunar/-/issues/1592 */
-      toplevel = gdk_window_get_toplevel (window);
-
       device = gdk_event_get_device (event);
 
       if (device != NULL && gdk_device_get_source (device) == GDK_SOURCE_KEYBOARD)
         device = gdk_device_get_associated_device (device);
 
       if (device != NULL)
-        gdk_window_get_device_position (toplevel, device, &rect.x, &rect.y, NULL);
+        {
+          /* Ensure that we have the toplevel window to work around a GTK3 Wayland bug
+           * that prevents the user from dismissing the menu via click in some cases.
+           * See: https://gitlab.xfce.org/xfce/thunar/-/issues/1592 */
+          window = gdk_window_get_toplevel (window);
+
+          gdk_window_get_device_position (window, device, &rect.x, &rect.y, NULL);
+        }
+
+      /* fallback needed for right-click DnD from xfdesktop to Thunar */
+      if (device == NULL)
+        {
+          display = gdk_window_get_display (window);
+          seat = gdk_display_get_default_seat (display);
+          device = gdk_seat_get_pointer (seat);
+
+          gdk_device_get_position (device, NULL, &rect.x, &rect.y);
+          window = gdk_device_get_window_at_position (device, &rect.x, &rect.y);
+        }
     }
 
   /* pop up the menu at the rectangle below the mouse cursor */
   gtk_menu_popup_at_rect (menu,
-                          toplevel,
+                          window,
                           &rect,
-                          GDK_GRAVITY_NORTH_WEST,
+                          GDK_GRAVITY_SOUTH_EAST,
                           GDK_GRAVITY_NORTH_WEST,
                           event);
 }
@@ -387,15 +407,16 @@ GMountOperation *
 thunar_gtk_mount_operation_new (gpointer parent)
 {
   GMountOperation *operation;
-  GdkScreen       *screen;
-  GtkWindow       *window = NULL;
+  GtkWindow       *window;
 
-  screen = thunar_util_parse_parent (parent, &window);
+  window = thunar_util_find_associated_window (parent);
 
   operation = gtk_mount_operation_new (window);
   g_mount_operation_set_password_save (G_MOUNT_OPERATION (operation), G_PASSWORD_SAVE_FOR_SESSION);
-  if (window == NULL && screen != NULL)
-    gtk_mount_operation_set_screen (GTK_MOUNT_OPERATION (operation), screen);
+
+  /* If a specific screen is requested, we spawn the dialog on that screen */
+  if (G_UNLIKELY (parent != NULL && GDK_IS_SCREEN (parent)))
+    gtk_mount_operation_set_screen (GTK_MOUNT_OPERATION (operation), GDK_SCREEN (parent));
 
   return operation;
 }
@@ -460,3 +481,15 @@ thunar_gtk_orientable_get_center_pos (GtkOrientable *orientable)
   else
     return (gint) (allocation.height / 2);
 }
+
+
+
+void
+thunar_gtk_window_set_screen (GtkWindow *window,
+                              GdkScreen *screen)
+{
+  /* If we want to run the dialog on a specific screen, we need to wipe the transient parent */
+  /* That is, because the window cannot be transient for a window on a different screen (workspace on that screen) */
+  gtk_window_set_transient_for (window, NULL);
+  gtk_window_set_screen (window, screen);
+}
