@@ -224,6 +224,9 @@ again:
                                                    _("The file \"%s\" already exists"),
                                                    display_name);
 
+              /* clean up */
+              g_free (display_name);
+
               /* check if we should overwrite */
               if (response == THUNAR_JOB_RESPONSE_REPLACE)
                 {
@@ -231,9 +234,6 @@ again:
                   if (_tij_delete_file (lp->data, thunar_job_get_cancellable (THUNAR_JOB (job)), &err))
                     goto again;
                 }
-
-              /* clean up */
-              g_free (display_name);
             }
           else
             {
@@ -402,6 +402,9 @@ again:
                                                    _("The file \"%s\" already exists"),
                                                    display_name);
 
+              /* clean up */
+              g_free (display_name);
+
               /* check if we should overwrite it */
               if (response == THUNAR_JOB_RESPONSE_REPLACE)
                 {
@@ -409,9 +412,6 @@ again:
                   if (_tij_delete_file (lp->data, thunar_job_get_cancellable (THUNAR_JOB (job)), &err))
                     goto again;
                 }
-
-              /* clean up */
-              g_free (display_name);
             }
           else
             {
@@ -1438,6 +1438,7 @@ _thunar_io_jobs_rename (ThunarJob *job,
   const gchar           *display_name;
   ThunarFile            *file;
   GError                *err = NULL;
+  GFile                 *old_file;
   gchar                 *old_file_uri;
   ThunarOperationLogMode log_mode;
   ThunarJobOperation    *operation;
@@ -1470,8 +1471,10 @@ _thunar_io_jobs_rename (ThunarJob *job,
       if (log_mode == THUNAR_OPERATION_LOG_OPERATIONS)
         {
           operation = thunar_job_operation_new (THUNAR_JOB_OPERATION_KIND_RENAME);
-          thunar_job_operation_add (operation, g_file_new_for_uri (old_file_uri), thunar_file_get_file (file));
+          old_file = g_file_new_for_uri (old_file_uri);
+          thunar_job_operation_add (operation, old_file, thunar_file_get_file (file));
           thunar_job_operation_history_commit (operation);
+          g_object_unref (old_file);
           g_object_unref (operation);
         }
     }
