@@ -76,6 +76,8 @@ thunar_path_entry_set_property (GObject      *object,
                                 guint         prop_id,
                                 const GValue *value,
                                 GParamSpec   *pspec);
+static void
+thunar_path_entry_realize (GtkWidget *widget);
 static gboolean
 thunar_path_entry_focus (GtkWidget       *widget,
                          GtkDirectionType direction);
@@ -107,6 +109,10 @@ thunar_path_entry_changed (GtkEditable *editable);
 static void
 thunar_path_entry_update_icon (ThunarPathEntry *path_entry);
 static void
+thunar_path_entry_notify_searching (ThunarPathEntry *path_entry,
+                                    GParamSpec      *pspec,
+                                    GtkWindow       *window);
+static void
 thunar_path_entry_do_insert_text (GtkEditable *editable,
                                   const gchar *new_text,
                                   gint         new_text_length,
@@ -153,6 +159,8 @@ struct _ThunarPathEntry
 {
   GtkEntry __parent__;
 
+  GtkWidget *window;
+
   ThunarIconFactory *icon_factory;
   ThunarFile        *current_folder;
   ThunarFile        *current_file;
@@ -166,6 +174,7 @@ struct _ThunarPathEntry
   guint check_completion_idle_id;
 
   gboolean search_mode;
+  gulong   notify_searching_handler_id;
 };
 
 
@@ -202,6 +211,7 @@ thunar_path_entry_class_init (ThunarPathEntryClass *klass)
   gobject_class->set_property = thunar_path_entry_set_property;
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
+  gtkwidget_class->realize = thunar_path_entry_realize;
   gtkwidget_class->focus = thunar_path_entry_focus;
   gtkwidget_class->drag_data_get = thunar_path_entry_drag_data_get;
   gtkwidget_class->drag_end = thunar_path_entry_drag_end;
@@ -303,6 +313,7 @@ thunar_path_entry_init (ThunarPathEntry *path_entry)
 
   /* disabled initially */
   path_entry->search_mode = FALSE;
+  path_entry->notify_searching_handler_id = 0;
 }
 
 
@@ -384,6 +395,19 @@ thunar_path_entry_set_property (GObject      *object,
 
 
 
+static void
+thunar_path_entry_realize (GtkWidget *widget)
+{
+  ThunarPathEntry *path_entry = THUNAR_PATH_ENTRY (widget);
+
+  (*GTK_WIDGET_CLASS (thunar_path_entry_parent_class)->realize) (widget);
+
+  /* toplevel window reference */
+  path_entry->window = gtk_widget_get_toplevel (widget);
+}
+
+
+
 static gboolean
 thunar_path_entry_focus (GtkWidget       *widget,
                          GtkDirectionType direction)
@@ -437,11 +461,11 @@ thunar_path_entry_icon_press_event (GtkEntry            *entry,
   gint             size;
   gint             scale_factor;
 
-  if (path_entry->current_file == NULL)
-    return FALSE;
-
   if (event->button == 1 && icon_pos == GTK_ENTRY_ICON_PRIMARY)
     {
+      if (path_entry->current_file == NULL)
+        return FALSE;
+
       /* save the drag button state */
       path_entry->drag_button = event->button;
 
@@ -473,6 +497,15 @@ thunar_path_entry_icon_press_event (GtkEntry            *entry,
       return TRUE;
     }
 
+  if (event->button == 1 && icon_pos == GTK_ENTRY_ICON_SECONDARY)
+    {
+      if (path_entry->search_mode == TRUE)
+        {
+          thunar_window_action_stop_search (THUNAR_WINDOW (path_entry->window));
+          return TRUE;
+        }
+    }
+
   return FALSE;
 }
 
@@ -500,12 +533,23 @@ thunar_path_entry_key_press_event (GtkWidget   *widget,
       /* we handled the event */
       return TRUE;
     }
-  /* cancel search with `Escape` */
-  if (G_UNLIKELY (path_entry->search_mode == TRUE && event->keyval == GDK_KEY_Escape && (event->state & GDK_CONTROL_MASK) == 0))
+
+  /* check if we are in search mode */
+  if (G_UNLIKELY (path_entry->search_mode == TRUE))
     {
-      GtkWidget *window = gtk_widget_get_toplevel (widget);
-      thunar_window_action_cancel_search (THUNAR_WINDOW (window));
-      return TRUE;
+      /* stop search with `Escape` */
+      if (event->keyval == GDK_KEY_Escape)
+        {
+          thunar_window_action_stop_search (THUNAR_WINDOW (path_entry->window));
+          return TRUE;
+        }
+
+      /* restart search with `Return` or `Enter` */
+      if (event->keyval == GDK_KEY_Return || event->keyval == GDK_KEY_KP_Enter)
+        {
+          thunar_window_update_search (THUNAR_WINDOW (path_entry->window));
+          return TRUE;
+        }
     }
 
   return FALSE;
@@ -627,19 +671,29 @@ thunar_path_entry_changed (GtkEditable *editable)
 
   if (G_UNLIKELY (text != NULL && thunar_util_is_a_search_query (text)) == TRUE)
     {
-      GtkWidget *window = gtk_widget_get_toplevel (GTK_WIDGET (editable));
-      path_entry->search_mode = TRUE;
+      thunar_window_update_search (THUNAR_WINDOW (path_entry->window));
+
+      if (path_entry->search_mode == FALSE)
+        {
+          path_entry->search_mode = TRUE;
+
+          /* update the path entry whenever there is an ongoing search */
+          path_entry->notify_searching_handler_id = g_signal_connect_swapped (G_OBJECT (path_entry->window), "notify::searching", G_CALLBACK (thunar_path_entry_notify_searching), path_entry);
+        }
+
       update_icon = TRUE;
-      if (GTK_IS_WINDOW (window))
-        thunar_window_update_search (THUNAR_WINDOW (window));
     }
   else
     {
       /* cancel search when part of 'Search: ' is deleted */
       if (path_entry->search_mode == TRUE)
+        thunar_window_cancel_search (THUNAR_WINDOW (path_entry->window));
+
+      /* disconnect the "notify::searching" signal handler */
+      if (path_entry->notify_searching_handler_id > 0)
         {
-          GtkWidget *window = gtk_widget_get_toplevel (GTK_WIDGET (editable));
-          thunar_window_action_cancel_search (THUNAR_WINDOW (window));
+          g_signal_handler_disconnect (G_OBJECT (path_entry->window), path_entry->notify_searching_handler_id);
+          path_entry->notify_searching_handler_id = 0;
         }
 
       /* location/folder-path code */
@@ -815,6 +869,50 @@ thunar_path_entry_update_icon (ThunarPathEntry *path_entry)
 
 
 
+static void
+thunar_path_entry_notify_searching (ThunarPathEntry *path_entry,
+                                    GParamSpec      *pspec,
+                                    GtkWindow       *window)
+{
+  gboolean searching;
+
+  _thunar_return_if_fail (THUNAR_IS_PATH_ENTRY (path_entry));
+  _thunar_return_if_fail (GTK_IS_WINDOW (window));
+
+  /* query the search status */
+  g_object_get (G_OBJECT (window), "searching", &searching, NULL);
+
+  /* setup the path entry to indicate an ongoing search */
+  if (path_entry->search_mode && searching)
+    {
+      ThunarPreferences *preferences;
+      gboolean           use_symbolic_icons;
+
+      preferences = thunar_preferences_get ();
+      g_object_get (G_OBJECT (preferences), "misc-symbolic-icons-in-toolbar", &use_symbolic_icons, NULL);
+      g_object_unref (G_OBJECT (preferences));
+
+      /* add the cancel button */
+      gtk_entry_set_icon_from_icon_name (GTK_ENTRY (path_entry), GTK_ENTRY_ICON_SECONDARY,
+                                         use_symbolic_icons ? "process-stop-symbolic" : "process-stop");
+      gtk_entry_set_icon_tooltip_text (GTK_ENTRY (path_entry), GTK_ENTRY_ICON_SECONDARY, _("Stop Search"));
+
+      /* add a tooltip */
+      gtk_widget_set_tooltip_text (GTK_WIDGET (path_entry), _("Search is in progress...\nPress the cancel button or the Escape key to stop."));
+    }
+  else
+    {
+      /* remove the cancel button */
+      gtk_entry_set_icon_from_icon_name (GTK_ENTRY (path_entry), GTK_ENTRY_ICON_SECONDARY, NULL);
+      gtk_entry_set_icon_tooltip_text (GTK_ENTRY (path_entry), GTK_ENTRY_ICON_SECONDARY, NULL);
+
+      /* remove the tooltip */
+      gtk_widget_set_tooltip_text (GTK_WIDGET (path_entry), NULL);
+    }
+}
+
+
+
 static void
 thunar_path_entry_do_insert_text (GtkEditable *editable,
                                   const gchar *new_text,
@@ -1413,24 +1511,43 @@ thunar_path_entry_cancel_search (ThunarPathEntry *path_entry)
 
 
 
+/**
+ * thunar_path_entry_get_search_mode:
+ * @path_entry : a #ThunarPathEntry.
+ *
+ * Return value: %TRUE if search mode is active, %FALSE otherwise.
+ **/
+gboolean
+thunar_path_entry_get_search_mode (ThunarPathEntry *path_entry)
+{
+  _thunar_return_val_if_fail (THUNAR_IS_PATH_ENTRY (path_entry), FALSE);
+
+  return path_entry->search_mode;
+}
+
+
+
 /**
  * thunar_path_entry_get_search_query:
  * @path_entry : a #ThunarPathEntry.
  *
- * Returns a copy of the search query in the text field of the @path_entry or an empty string
- * if the @path_entry doesn't contain a search query.
+ * Returns a copy of the search query in the text field of the @path_entry, an empty string
+ * if the @path_entry doesn't contain a search query or %NULL if not in search mode.
  *
  * It's the responsibility of the caller to free the returned string using `g_free`.
  **/
 gchar *
 thunar_path_entry_get_search_query (ThunarPathEntry *path_entry)
 {
-  unsigned long search_prefix_length;
-  const gchar  *text = gtk_entry_get_text (GTK_ENTRY (path_entry));
-  search_prefix_length = strlen (thunar_util_get_search_prefix ());
+  const gchar  *text;
+  unsigned long search_prefix_length = strlen (thunar_util_get_search_prefix ());
 
   _thunar_return_val_if_fail (THUNAR_IS_PATH_ENTRY (path_entry), NULL);
-  _thunar_return_val_if_fail (strncmp (text, thunar_util_get_search_prefix (), search_prefix_length) == 0, NULL);
+
+  text = gtk_entry_get_text (GTK_ENTRY (path_entry));
+
+  if (strncmp (text, thunar_util_get_search_prefix (), search_prefix_length) != 0)
+    return NULL;
 
   return strlen (text) > search_prefix_length ? g_strdup (&text[search_prefix_length]) : g_strdup ("");
 }
