@@ -286,7 +286,7 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   xfce_titled_dialog_add_action_widget (XFCE_TITLED_DIALOG (dialog), button, GTK_RESPONSE_HELP);
   gtk_widget_show (button);
 
-  notebook = gtk_notebook_new ();
+  notebook = xfce_notebook_new ();
   gtk_container_set_border_width (GTK_CONTAINER (notebook), 6);
   gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), notebook, TRUE, TRUE, 0);
   gtk_widget_show (notebook);
@@ -395,7 +395,7 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
                           "active",
                           G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
   gtk_widget_set_tooltip_text (button,
-                               _("Select this option to remember view type, zoom level, sort column, and sort order individually for each folder"));
+                               _("Select this option to remember view type, zoom level and sort settings individually for each folder"));
   gtk_widget_set_hexpand (button, TRUE);
   gtk_grid_attach (GTK_GRID (grid), button, 0, row, 1, 1);
   gtk_widget_show (button);
